from enum import Enum


class QuoteField(str, Enum):
    """ Enum of all valid fields on Quote object """
    # ------ Quote ---
    LASTPRICE = "LASTPRICE"
    DAY_OPEN = "DAY_OPEN"
    DAY_HIGH = "DAY_HIGH"
    DAY_LOW = "DAY_LOW"
    DAY_VOLUME = "DAY_VOLUME"
    PREV_DAY_CLOSE = "PREV_DAY_CLOSE"
    DAY_CHANGE = "DAY_CHANGE"
    DAY_CHANGE_PCT = "DAY_CHANGE_PCT"
    DAY_CHANGE_OPEN = "DAY_CHANGE_OPEN"

class FinancialField(str, Enum):
    """Financial Fields supported by Invesfly"""

    # SHARE
    OUTSTANDING_SHARES = "OUTSTANDING_SHARES"
    MARKET_CAP = "MARKET_CAP"

    # INCOME STATEMENT
    REVENUE = "REVENUE"
    COST_OF_REVENUE = "COST_OF_REVENUE"
    GROSS_PROFIT = "GROSS_PROFIT"
    OPERATING_EXPENSE = "OPERATING_EXPENSE"
    OPERATING_INCOME = "OPERATING_INCOME"
    NET_INCOME = "NET_INCOME"  # subtract taxes to get net income
    EPS = "EPS"

    # BALANCE SHEET: Assets = Liabilities + Equity
    CURR_ASSETS = "CURR_ASSETS"
    NON_CURR_ASSETS = "NON_CURR_ASSETS"
    ASSETS = "ASSETS"
    CURR_LIABILITIES = "CURR_LIABILITIES"
    NON_CURR_LIABILITIES = "NON_CURR_LIABILITIES"
    LIABILITIES = "LIABILITIES"
    LONG_TERM_DEBT = "LONG_TERM_DEBT"
    EQUITY = "EQUITY"

    # CASH FLOW
    OPEARTING_CASH_FLOW = "OPEARTING_CASH_FLOW"
    INVESTING_CASH_FLOW = "INVESTING_CASH_FLOW"
    FINANCING_CASH_FLOW = "FINANCING_CASH_FLOW"
    NET_CASH_FLOW = "NET_CASH_FLOW"

    RETURN_ON_EQUITY = "RETURN_ON_EQUITY"
    RETURN_ON_ASSETS = "RETURN_ON_ASSETS"
    PRICE_TO_EARNINGS_RATIO = "PRICE_TO_EARNINGS_RATIO"
    PEG_RATIO = "PEG_RATIO"
    PRICE_TO_SALES_RATIO = "PRICE_TO_SALES_RATIO"
    PRICE_TO_BOOK_RATIO = "PRICE_TO_BOOK_RATIO"
    REVENUE_GROWTH = "REVENUE_GROWTH"
    EPS_GROWTH = "EPS_GROWTH"
    DEBT_EQUITY_RATIO = "DEBT_EQUITY_RATIO"
    CURRENT_RATIO = "CURRENT_RATIO"
    GROSS_MARGIN = "GROSS_MARGIN"
    PROFIT_MARGIN = "PROFIT_MARGIN"

    DIVIDEND_YIELD = "DIVIDEND_YIELD"


class StandardIndicatorId(str, Enum):
    """ Technical Indicator supported by Investfly"""

    # Works by default on DAILY bars
    AVGVOL = "AVGVOLUME"
    HIGH_52_WEEK = "HIGH52WEEK"
    LOW_52_WEEK = "LOW52WEEK"

    DRAWDOWN = "DRAWDOWN"
    PERCENTCHANGE = "PRICECHANGEPCT"

    SMA = "SMA"
    EMA = "EMA"
    MACD = "MACD"
    MACDS = "MACDS"
    RSI = "RSI"
    ATR = "ATR"
    ADX = "ADX"
    PLUS_DI = "PLUS_DI"
    MINUS_DI = "MINUS_DI"
    PSAR = "PSAR"
    STD_DEV = "STD_DEV"

    BBAND = "BBAND"  # composite, only for chart
    UPPER_BBAND = "UPPER_BBAND"
    LOWER_BBAND = "LOWER_BBAND"
    WILLIAM_R = "WILLIAM_R"
    ROC = "ROC"
    CCI = "CCI"

    CMO = "CMO"
    CMO_SMOOTHED = "CMO_SMOOTHED"
    MEDIAN_PRICE = "MEDIAN_PRICE"
    TYPICAL_PRICE = "TYPICAL_PRICE"
    ULTIMATE_OSC = "ULTIMATE_OSC"
    PPO = "PPO"

    STOCHASTICS = "STOCHASTICS"  # composite only for charts
    FAST_STOCHASTIC_OSC = "FAST_STOCHASTIC_OSC"
    SLOW_STOCHASTIC_OSC = "SLOW_STOCHASTIC_OSC"

    MAX = "MAX"
    MIN = "MIN"

    DOJI = "DOJI"
    HAMMER = "HAMMER"

    INVERTED_HAMMER = "INVERTED_HAMMER"
    DRAGONFLY_DOJI = "DRAGONFLY_DOJI"
    GRAVESTONE_DOJI = "GRAVESTONE_DOJI"
    HANGING_MAN = "HANGING_MAN"

    BULLISH = "BULLISH"
    BEARISH = "BEARISH"

    SUPPORT = "SUPPORT"
    RESISTANCE = "RESISTANCE"
